==> brumbrumrally_0.6.orig.tar.gz: brumbrumrally-0.6/src/Settings.cpp <==
/*
 * Copyright 2014, 2016 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "Settings.h"
#include "TileSet.h"
#include "str.h"
#include <sstream>
#include <cstdlib>
#include <cstring>
#ifdef _WIN32
#include <windows.h>
#endif

Settings::Graphics Settings::graphics;
Settings::Players Settings::players;
Settings::Network Settings::network;
Settings::Tracks Settings::tracks;
Settings::Race Settings::race;
Settings::TimeTrial Settings::timetrial;

template <typename T>
void read(std::istream& in, T& value, T min, T max)
{
	std::string line;
	if (std::getline(in, line))
	{
		T input;
		std::istringstream iss(line);
		if (iss >> input && input >= min && input <= max)
		{
			value = input;
		}
	}
}

void read(std::istream& in, std::string& value)
{
	std::string line;
	if (std::getline(in, line))
	{
		for (std::size_t i = 0; i < line.size(); ++i)
		{
			if (std::isgraph(line[i]))
			{
				value = line;
				break;
			}
		}
	}
}

template <typename T>
void write(std::ostream& out, T value)
{
	out << value << '\n';
}

Settings::Graphics::Graphics()
:	scaler(0),
	fullscreen(0),
	fullscreenWidth(0),
	fullscreenHeight(0),
	stretch(0),
	keepAspectRatio(1),
	pointer("standard"),
	scalePointer(1)
{
}

void Settings::Graphics::save(std::ostream& out)
{
	write(out, scaler);
	write(out, fullscreen);
	write(out, fullscreenWidth);
	write(out, fullscreenHeight);
	write(out, stretch);
	write(out, keepAspectRatio);
	write(out, pointer);
	write(out, scalePointer);
}

void Settings::Graphics::load(std::istream& in)
{
	read(in, scaler, 0, 3);
	read(in, fullscreen, 0, 1);
	read(in, fullscreenWidth, 0, 10000);
	read(in, fullscreenHeight, 0, 10000);
	read(in, stretch, 0, 1);
	read(in, keepAspectRatio, 0, 1);
	read(in, pointer);
	read(in, scalePointer, 0, 2);
}

Settings::Players::Players()
{
	const char* username = 0;
	const std::size_t USERNAME_MAX_LENGTH = 12;
#ifdef _WIN32
	const int BUFFER_LENGTH = USERNAME_MAX_LENGTH + 1;
	char buffer[BUFFER_LENGTH];
	DWORD bufferSize = BUFFER_LENGTH;
	if (GetUserName(buffer, &bufferSize))
	{
		username = buffer;
	}
#else
	username = std::getenv("USER");
	if (!username)
	{
		username = std::getenv("USERNAME");
		if (!username)
		{
			username = std::getenv("LOGNAME");
		}
	}	
#endif
	if (!username || !*username || std::strlen(username) > USERNAME_MAX_LENGTH || std::strcmp(username, "root") == 0)
	{
		username = "player 1";
	}
	humanNames[0] = username;
	toLower(humanNames[0]);
	
	for (int i = 1; i < 8; ++i)
	{
		std::ostringstream oss;
		oss << "player " << (i + 1);
		humanNames[i] = oss.str();
	}
	
	for (int i = 0; i < 8; ++i)
	{
		std::ostringstream oss;
		oss << "computer " << (i + 1);
		computerNames[i] = oss.str();
		computerDifficulties[i] = -1;
	}
}

void Settings::Players::save(std::ostream& out)
{
	for (int i = 0; i < 8; ++i)
	{
		write(out, humanNames[i]);
	}
	for (int i = 0; i < 8; ++i)
	{
		write(out, computerNames[i]);
	}
	for (int i = 0; i < 8; ++i)
	{
		write(out, computerDifficulties[i]);
	}
}

void Settings::Players::load(std::istream& in)
{
	for (int i = 0; i < 8; ++i)
	{
		read(in, humanNames[i]);
	}
	for (int i = 0; i < 8; ++i)
	{
		read(in, computerNames[i]);
	}
	for (int i = 0; i < 8; ++i)
	{
		read(in, computerDifficulties[i], -1, 4);
	}
}

Settings::Network::Network()
:	players(1),
	network(0),
	maxPlayers(8),
	maxPlayersPerClient(4),
	computerPlayers(0),
	computerDifficulty(2),
	laps(3),
	tracks(4),
	collisions(true),
	tournament(false),
	server("dataapa.net"),
	maxConnections(50),
	chatFont(),
	maxChatMessages(10),
	chatMessageDuration(30)
{
	for (int port = DEFAULT_LAN_PORTS_FIRST; port <= DEFAULT_LAN_PORTS_LAST; ++port)
	{
		lanPorts.push_back(port);
	}
}

void Settings::Network::save(std::ostream& out)
{
	write(out, players);
	write(out, network);
	write(out, name);
	write(out, maxPlayers);
	write(out, maxPlayersPerClient);
	write(out, computerPlayers);
	write(out, computerDifficulty);
	write(out, laps);
	write(out, tracks);
	write(out, collisions);
	
	for (std::size_t i = 0; i < internetPorts.size(); ++i)
	{
		out << internetPorts[i] << ' ';
	}
	out << '\n';
	
	for (std::size_t i = 0; i < lanPorts.size(); ++i)
	{
		out << lanPorts[i] << ' ';
	}
	out << '\n';
	
	write(out, server);
	write(out, maxConnections);
	write(out, chatFont);
	write(out, maxChatMessages);
	write(out, chatMessageDuration);
}

void Settings::Network::load(std::istream& in)
{
	read(in, players, 1, 7);
	read(in, network, 0, 1);
	read(in, name);
	read(in, maxPlayers, 2, 8);
	read(in, maxPlayersPerClient, 1, 7);
	read(in, computerPlayers, 0, 6);
	read(in, computerDifficulty, 0, 4);
	read(in, laps, 1, 9);
	read(in, tracks, 2, 99);
	read(in, collisions, 0, 1);
	
	std::string line;
	std::getline(in, line);
	std::istringstream iss(line);
	std::vector<Uint16> ports;
	Uint16 port;

	while (iss >> port && port != 0)
	{
		ports.push_back(port);
	}
	if (!ports.empty())
	{
		internetPorts = ports;
	}
	
	std::getline(in, line);
	iss.str(line);
	ports.clear();
	
	while (iss >> port && port != 0)
	{
		ports.push_back(port);
	}
	if (!ports.empty())
	{
		lanPorts = ports;
	}
	
	read(in, server);
	read(in, maxConnections, 1U, 1000U);
	read(in, chatFont, 0, 1);
	read(in, maxChatMessages, 1U, 20U);
	read(in, chatMessageDuration, -1, 100000);
}

Settings::Tracks::Tracks()
:	minLength(10), // Tracks need to have at least length 10 to be interresting and to avoid similarity.
	maxLength(MAX_TRACK_LENGTH),
	minWidth(MIN_TRACK_WIDTH),
	maxWidth(MAX_TRACK_WIDTH),
	minHeight(MIN_TRACK_HEIGHT),
	maxHeight(MAX_TRACK_HEIGHT),
	minCorners(MIN_TRACK_CORNERS),
	maxCorners(MAX_TRACK_CORNERS),
	minLongestStraighaway(MIN_LONGEST_STRAIGHTAWAY),
	maxLongestStraighaway(MAX_LONGEST_STRAIGHTAWAY),
	minLongestCornerSeries(MIN_LONGEST_CORNER_SERIES),
	maxLongestCornerSeries(MAX_LONGEST_CORNER_SERIES)
{
	const std::vector<std::string>& sets = TileSet::getTileSetNames();
	std::vector<std::string>::const_iterator it;
	for (it = sets.begin(); it != sets.end(); ++it)
	{
		tilesets[*it] = 1;
	}
}

void Settings::Tracks::save(std::ostream& out)
{
	write(out, minLength);
	write(out, maxLength);
	write(out, minWidth);
	write(out, maxWidth);
	write(out, minHeight);
	write(out, maxHeight);
	write(out, minCorners);
	write(out, maxCorners);
	write(out, minLongestStraighaway);
	write(out, maxLongestStraighaway);
	write(out, minLongestCornerSeries);
	write(out, maxLongestCornerSeries);
	
	std::map<std::string, int>::iterator it;
	for (it = tilesets.begin(); it != tilesets.end(); ++it)
	{
		out << it->first << ' ' << it->second << ' ';
	}
	out << '\n';
}

void Settings::Tracks::load(std::istream& in)
{
	read(in, minLength, MIN_TRACK_LENGTH, MAX_TRACK_LENGTH);
	read(in, maxLength, MIN_TRACK_LENGTH, MAX_TRACK_LENGTH);
	if (minLength > maxLength)
	{
		std::swap(minLength, maxLength);
	}
	
	read(in, minWidth, MIN_TRACK_WIDTH, MAX_TRACK_WIDTH);
	read(in, maxWidth, MIN_TRACK_WIDTH, MAX_TRACK_WIDTH);
	if (minWidth > maxWidth)
	{
		std::swap(minWidth, maxWidth);
	}
	
	read(in, minHeight, MIN_TRACK_HEIGHT, MAX_TRACK_HEIGHT);
	read(in, maxHeight, MIN_TRACK_HEIGHT, MAX_TRACK_HEIGHT);
	if (minHeight > maxHeight)
	{
		std::swap(minHeight, maxHeight);
	}
	
	read(in, minCorners, MIN_TRACK_CORNERS, MAX_TRACK_CORNERS);
	read(in, maxCorners, MIN_TRACK_CORNERS, MAX_TRACK_CORNERS);
	if (minCorners > maxCorners)
	{
		std::swap(minCorners, maxCorners);
	}
	
	read(in, minLongestStraighaway, MIN_LONGEST_STRAIGHTAWAY, MAX_LONGEST_STRAIGHTAWAY);
	read(in, maxLongestStraighaway, MIN_LONGEST_STRAIGHTAWAY, MAX_LONGEST_STRAIGHTAWAY);
	if (minLongestStraighaway > maxLongestStraighaway)
	{
		std::swap(minLongestStraighaway, maxLongestStraighaway);
	}
	
	read(in, minLongestCornerSeries, MIN_LONGEST_CORNER_SERIES, MAX_LONGEST_CORNER_SERIES);
	read(in, maxLongestCornerSeries, MIN_LONGEST_CORNER_SERIES, MAX_LONGEST_CORNER_SERIES);
	if (minLongestCornerSeries > maxLongestCornerSeries)
	{
		std::swap(minLongestCornerSeries, maxLongestCornerSeries);
	}
	
	std::string line;
	std::getline(in, line);
	std::istringstream iss(line);
	
	std::string tilesetName;
	int tilesetPercentage;
	while (iss >> tilesetName >> tilesetPercentage)
	{
		std::map<std::string, int>::iterator it = tilesets.find(tilesetName);
		if(it != tilesets.end() && tilesetPercentage >= 0 && tilesetPercentage <= 10000)
		{			
			it->second = tilesetPercentage;
		}
	}
}

Settings::Race::Race()
:	humanPlayers(1),
	computerPlayers(3),
	laps(3),
	tracks(4),
	collisions(true),
	difficulty(2),
	tournament(false)
{
}

void Settings::Race::save(std::ostream& out)
{
	write(out, humanPlayers);
	write(out, computerPlayers);
	write(out, laps);
	write(out, tracks);
	write(out, collisions);
	write(out, difficulty);
}

void Settings::Race::load(std::istream& in)
{
	read(in, humanPlayers, 0, 8);
	read(in, computerPlayers, 0, 8);
	read(in, laps, 1, 9);
	read(in, tracks, 2, 99);
	read(in, collisions, 0, 1);
	read(in, difficulty, 0, 4);
}

Settings::TimeTrial::TimeTrial()
:	car(0),
	remember(0),
	showDriverGhost(1),
	showBestGhost(1),
	laps(1)
{
}

void Settings::TimeTrial::save(std::ostream& out)
{
	write(out, name);
	write(out, remember);
	write(out, showDriverGhost);
	write(out, showBestGhost);
	write(out, laps);
}

void Settings::TimeTrial::load(std::istream& in)
{
	read(in, name);
	read(in, remember, 0, 1);
	read(in, showDriverGhost, 0, 1);
	read(in, showBestGhost, 0, 1);
	read(in, laps, 1, 9);
}

void Settings::save(std::ostream& out)
{
	Settings::graphics.save(out);
	Settings::players.save(out);
	Settings::network.save(out);
	Settings::tracks.save(out);
	Settings::race.save(out);
	Settings::timetrial.save(out);
}

void Settings::load(std::istream& in)
{
	Settings::graphics.load(in);
	Settings::players.load(in);
	Settings::network.load(in);
	Settings::tracks.load(in);
	Settings::race.load(in);
	Settings::timetrial.load(in);
}

==> brumbrumrally-0.7.tar.gz: brumbrumrally-0.7/src/Settings.cpp <==
/*
 * Copyright 2014, 2016 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "Settings.h"
#include "TileSet.h"
#include "str.h"
#include <cctype>
#include <cstdlib>
#include <cstring>
#include <sstream>
#ifdef _WIN32
#include <windows.h>
#endif

Settings::Graphics Settings::graphics;
Settings::Players Settings::players;
Settings::Network Settings::network;
Settings::Tracks Settings::tracks;
Settings::Race Settings::race;
Settings::TimeTrial Settings::timetrial;

template <typename T>
void read(std::istream& in, T& value, T min, T max)
{
	std::string line;
	if (std::getline(in, line))
	{
		T input;
		std::istringstream iss(line);
		if (iss >> input && input >= min && input <= max)
		{
			value = input;
		}
	}
}

void read(std::istream& in, std::string& value)
{
	std::string line;
	if (std::getline(in, line))
	{
		for (std::size_t i = 0; i < line.size(); ++i)
		{
			if (std::isgraph(line[i]))
			{
				value = line;
				break;
			}
		}
	}
}

template <typename T>
void write(std::ostream& out, T value)
{
	out << value << '\n';
}

Settings::Graphics::Graphics()
:	scaler(0),
	fullscreen(0),
	fullscreenWidth(0),
	fullscreenHeight(0),
	stretch(0),
	keepAspectRatio(1),
	pointer("standard"),
	scalePointer(1)
{
}

void Settings::Graphics::save(std::ostream& out)
{
	write(out, scaler);
	write(out, fullscreen);
	write(out, fullscreenWidth);
	write(out, fullscreenHeight);
	write(out, stretch);
	write(out, keepAspectRatio);
	write(out, pointer);
	write(out, scalePointer);
}

void Settings::Graphics::load(std::istream& in)
{
	read(in, scaler, 0, 3);
	read(in, fullscreen, 0, 1);
	read(in, fullscreenWidth, 0, 10000);
	read(in, fullscreenHeight, 0, 10000);
	read(in, stretch, 0, 1);
	read(in, keepAspectRatio, 0, 1);
	read(in, pointer);
	read(in, scalePointer, 0, 2);
}

Settings::Players::Players()
{
	const char* username = 0;
	const std::size_t USERNAME_MAX_LENGTH = 12;
#ifdef _WIN32
	const int BUFFER_LENGTH = USERNAME_MAX_LENGTH + 1;
	char buffer[BUFFER_LENGTH];
	DWORD bufferSize = BUFFER_LENGTH;
	if (GetUserName(buffer, &bufferSize))
	{
		username = buffer;
	}
#else
	username = std::getenv("USER");
	if (!username)
	{
		username = std::getenv("USERNAME");
		if (!username)
		{
			username = std::getenv("LOGNAME");
		}
	}	
#endif
	if (!username || !*username || std::strlen(username) > USERNAME_MAX_LENGTH || std::strcmp(username, "root") == 0)
	{
		username = "player 1";
	}
	humanNames[0] = username;
	toLower(humanNames[0]);
	
	for (int i = 1; i < 8; ++i)
	{
		std::ostringstream oss;
		oss << "player " << (i + 1);
		humanNames[i] = oss.str();
	}
	
	for (int i = 0; i < 8; ++i)
	{
		std::ostringstream oss;
		oss << "computer " << (i + 1);
		computerNames[i] = oss.str();
		computerDifficulties[i] = -1;
	}
}

void Settings::Players::save(std::ostream& out)
{
	for (int i = 0; i < 8; ++i)
	{
		write(out, humanNames[i]);
	}
	for (int i = 0; i < 8; ++i)
	{
		write(out, computerNames[i]);
	}
	for (int i = 0; i < 8; ++i)
	{
		write(out, computerDifficulties[i]);
	}
}

void Settings::Players::load(std::istream& in)
{
	for (int i = 0; i < 8; ++i)
	{
		read(in, humanNames[i]);
	}
	for (int i = 0; i < 8; ++i)
	{
		read(in, computerNames[i]);
	}
	for (int i = 0; i < 8; ++i)
	{
		read(in, computerDifficulties[i], -1, 4);
	}
}

Settings::Network::Network()
:	players(1),
	network(0),
	maxPlayers(8),
	maxPlayersPerClient(4),
	computerPlayers(0),
	computerDifficulty(2),
	laps(3),
	tracks(4),
	collisions(true),
	tournament(false),
	server("dataapa.net"),
	maxConnections(50),
	chatFont(),
	maxChatMessages(10),
	chatMessageDuration(30)
{
	for (int port = DEFAULT_LAN_PORTS_FIRST; port <= DEFAULT_LAN_PORTS_LAST; ++port)
	{
		lanPorts.push_back(port);
	}
}

void Settings::Network::save(std::ostream& out)
{
	write(out, players);
	write(out, network);
	write(out, name);
	write(out, maxPlayers);
	write(out, maxPlayersPerClient);
	write(out, computerPlayers);
	write(out, computerDifficulty);
	write(out, laps);
	write(out, tracks);
	write(out, collisions);
	
	for (std::size_t i = 0; i < internetPorts.size(); ++i)
	{
		out << internetPorts[i] << ' ';
	}
	out << '\n';
	
	for (std::size_t i = 0; i < lanPorts.size(); ++i)
	{
		out << lanPorts[i] << ' ';
	}
	out << '\n';
	
	write(out, server);
	write(out, maxConnections);
	write(out, chatFont);
	write(out, maxChatMessages);
	write(out, chatMessageDuration);
}

void Settings::Network::load(std::istream& in)
{
	read(in, players, 1, 7);
	read(in, network, 0, 1);
	read(in, name);
	read(in, maxPlayers, 2, 8);
	read(in, maxPlayersPerClient, 1, 7);
	read(in, computerPlayers, 0, 6);
	read(in, computerDifficulty, 0, 4);
	read(in, laps, 1, 9);
	read(in, tracks, 2, 99);
	read(in, collisions, 0, 1);
	
	std::string line;
	std::getline(in, line);
	std::istringstream iss(line);
	std::vector<Uint16> ports;
	Uint16 port;

	while (iss >> port && port != 0)
	{
		ports.push_back(port);
	}
	if (!ports.empty())
	{
		internetPorts = ports;
	}
	
	std::getline(in, line);
	iss.str(line);
	ports.clear();
	
	while (iss >> port && port != 0)
	{
		ports.push_back(port);
	}
	if (!ports.empty())
	{
		lanPorts = ports;
	}
	
	read(in, server);
	read(in, maxConnections, 1U, 1000U);
	read(in, chatFont, 0, 1);
	read(in, maxChatMessages, 1U, 20U);
	read(in, chatMessageDuration, -1, 100000);
}

Settings::Tracks::Tracks()
:	minLength(10), // Tracks need to have at least length 10 to be interesting and to avoid similarity.
	maxLength(MAX_TRACK_LENGTH),
	minWidth(MIN_TRACK_WIDTH),
	maxWidth(MAX_TRACK_WIDTH),
	minHeight(MIN_TRACK_HEIGHT),
	maxHeight(MAX_TRACK_HEIGHT),
	minCorners(MIN_TRACK_CORNERS),
	maxCorners(MAX_TRACK_CORNERS),
	minLongestStraighaway(MIN_LONGEST_STRAIGHTAWAY),
	maxLongestStraighaway(MAX_LONGEST_STRAIGHTAWAY),
	minLongestCornerSeries(MIN_LONGEST_CORNER_SERIES),
	maxLongestCornerSeries(MAX_LONGEST_CORNER_SERIES)
{
	const std::vector<std::string>& sets = TileSet::getTileSetNames();
	std::vector<std::string>::const_iterator it;
	for (it = sets.begin(); it != sets.end(); ++it)
	{
		tilesets[*it] = 1;
	}
}

void Settings::Tracks::save(std::ostream& out)
{
	write(out, minLength);
	write(out, maxLength);
	write(out, minWidth);
	write(out, maxWidth);
	write(out, minHeight);
	write(out, maxHeight);
	write(out, minCorners);
	write(out, maxCorners);
	write(out, minLongestStraighaway);
	write(out, maxLongestStraighaway);
	write(out, minLongestCornerSeries);
	write(out, maxLongestCornerSeries);
	
	std::map<std::string, int>::iterator it;
	for (it = tilesets.begin(); it != tilesets.end(); ++it)
	{
		out << it->first << ' ' << it->second << ' ';
	}
	out << '\n';
}

void Settings::Tracks::load(std::istream& in)
{
	read(in, minLength, MIN_TRACK_LENGTH, MAX_TRACK_LENGTH);
	read(in, maxLength, MIN_TRACK_LENGTH, MAX_TRACK_LENGTH);
	if (minLength > maxLength)
	{
		std::swap(minLength, maxLength);
	}
	
	read(in, minWidth, MIN_TRACK_WIDTH, MAX_TRACK_WIDTH);
	read(in, maxWidth, MIN_TRACK_WIDTH, MAX_TRACK_WIDTH);
	if (minWidth > maxWidth)
	{
		std::swap(minWidth, maxWidth);
	}
	
	read(in, minHeight, MIN_TRACK_HEIGHT, MAX_TRACK_HEIGHT);
	read(in, maxHeight, MIN_TRACK_HEIGHT, MAX_TRACK_HEIGHT);
	if (minHeight > maxHeight)
	{
		std::swap(minHeight, maxHeight);
	}
	
	read(in, minCorners, MIN_TRACK_CORNERS, MAX_TRACK_CORNERS);
	read(in, maxCorners, MIN_TRACK_CORNERS, MAX_TRACK_CORNERS);
	if (minCorners > maxCorners)
	{
		std::swap(minCorners, maxCorners);
	}
	
	read(in, minLongestStraighaway, MIN_LONGEST_STRAIGHTAWAY, MAX_LONGEST_STRAIGHTAWAY);
	read(in, maxLongestStraighaway, MIN_LONGEST_STRAIGHTAWAY, MAX_LONGEST_STRAIGHTAWAY);
	if (minLongestStraighaway > maxLongestStraighaway)
	{
		std::swap(minLongestStraighaway, maxLongestStraighaway);
	}
	
	read(in, minLongestCornerSeries, MIN_LONGEST_CORNER_SERIES, MAX_LONGEST_CORNER_SERIES);
	read(in, maxLongestCornerSeries, MIN_LONGEST_CORNER_SERIES, MAX_LONGEST_CORNER_SERIES);
	if (minLongestCornerSeries > maxLongestCornerSeries)
	{
		std::swap(minLongestCornerSeries, maxLongestCornerSeries);
	}
	
	std::string line;
	std::getline(in, line);
	std::istringstream iss(line);
	
	std::string tilesetName;
	int tilesetPercentage;
	while (iss >> tilesetName >> tilesetPercentage)
	{
		std::map<std::string, int>::iterator it = tilesets.find(tilesetName);
		if(it != tilesets.end() && tilesetPercentage >= 0 && tilesetPercentage <= 10000)
		{			
			it->second = tilesetPercentage;
		}
	}
}

Settings::Race::Race()
:	humanPlayers(1),
	computerPlayers(3),
	laps(3),
	tracks(4),
	collisions(true),
	difficulty(2),
	tournament(false)
{
}

void Settings::Race::save(std::ostream& out)
{
	write(out, humanPlayers);
	write(out, computerPlayers);
	write(out, laps);
	write(out, tracks);
	write(out, collisions);
	write(out, difficulty);
}

void Settings::Race::load(std::istream& in)
{
	read(in, humanPlayers, 0, 8);
	read(in, computerPlayers, 0, 8);
	read(in, laps, 1, 9);
	read(in, tracks, 2, 99);
	read(in, collisions, 0, 1);
	read(in, difficulty, 0, 4);
}

Settings::TimeTrial::TimeTrial()
:	car(0),
	remember(0),
	showDriverGhost(1),
	showBestGhost(1),
	laps(1)
{
}

void Settings::TimeTrial::save(std::ostream& out)
{
	write(out, name);
	write(out, remember);
	write(out, showDriverGhost);
	write(out, showBestGhost);
	write(out, laps);
}

void Settings::TimeTrial::load(std::istream& in)
{
	read(in, name);
	read(in, remember, 0, 1);
	read(in, showDriverGhost, 0, 1);
	read(in, showBestGhost, 0, 1);
	read(in, laps, 1, 9);
}

void Settings::save(std::ostream& out)
{
	Settings::graphics.save(out);
	Settings::players.save(out);
	Settings::network.save(out);
	Settings::tracks.save(out);
	Settings::race.save(out);
	Settings::timetrial.save(out);
}

void Settings::load(std::istream& in)
{
	Settings::graphics.load(in);
	Settings::players.load(in);
	Settings::network.load(in);
	Settings::tracks.load(in);
	Settings::race.load(in);
	Settings::timetrial.load(in);
}
